@@ -1,4 +1,4 @@
-# Copyright (C) 2011 Eric Leblond <eric@regit.org>
+# Copyright (C) 2011,2012 Eric Leblond <eric@regit.org>
 #
 # You can copy, redistribute or modify this Program under the terms of
 # the GNU General Public License version 3 as published by the Free
@@ -23,6 +23,8 @@ from tempfile import NamedTemporaryFile
 import errno
 import re
 
+COCCIGREP_VERSION = "1.11"
+
 have_multiprocessing = True
 try:
     from multiprocessing import Process, Pipe
@@ -307,13 +309,13 @@ for p in p1:
             op = _operation_name(fname)
             self.operations[op] = path.join(self.get_datadir(), fname)
 
-    def setup(self, stype, attribut, operation):
+    def setup(self, stype, attribute, operation):
         """
         :param stype: structure name, used to replace '$type' in the cocci file
         :type stype: str
-        :param attribut: basically attribut of the structure, used to replace
-            '$attribut' in the cocci file
-        :type attribut: str
+        :param attribute: basically attribute of the structure, used to replace
+            '$attribute' in the cocci file
+        :type attribute: str
         :param operation: search operation to do
         :type operation: str
         :raise: :class:`CocciRunException`
@@ -322,7 +324,7 @@ for p in p1:
             raise CocciRunException("Can't use coccigrep without type to "
                 "search")
         self.type = stype
-        self.attribut = attribut
+        self.attribute = attribute
         self.operation = operation
 
     def set_concurrency(self, ncpus):
@@ -402,27 +404,40 @@ for p in p1:
         """
         self.verbose = True
 
+    def get_spatch_version(self):
+        cmd = [self.spatch] + [ '--version']
+        try:
+            output = Popen(cmd, stderr=PIPE).communicate()[1]
+        except OSError, err:
+            unlink(tmp_cocci_file_name)
+            _raise_run_err(err, cmd)
+        reg = r"version (.*?) with"
+        m = re.search(reg, output)
+        return m.group(1)
+
+    def spatch_newer_than(self, version):
+        from distutils.version import LooseVersion
+        sversion = self.get_spatch_version()
+        return LooseVersion(sversion) > LooseVersion(version)
+
     def run(self, files):
         """
-        Run the search against the files given in argument
+        Run the search against the files and directories given in argument
 
         This function is doing the main job. It will run spatch with
         the correct parameters by using subprocess or it will use
         multiprocessing if a concurrency level greater than 1 has been
         asked.
 
-        :param files: list of filenames
-        :type files: list of str
+        :param args: list of filenames and directory names
+        :type args: list of str
         :raise: :class:`CocciRunException` or :class:`CocciConfigException`
         """
 
         if len(files) == 0:
             raise CocciRunException("Can't use coccigrep without files "
                 "to search")
-        for cfile in files:
-            if not path.isfile(cfile):
-                raise CocciRunException("'%s' is not a file, can't "
-                    "continue" % cfile)
+
         # create tmp cocci file:
         tmp_cocci_file = NamedTemporaryFile(suffix=".cocci")
         tmp_cocci_file_name = tmp_cocci_file.name
@@ -432,8 +447,14 @@ for p in p1:
         cocci_tmpl = cocci_file.read()
         cocci_smpl_tmpl = Template(cocci_tmpl)
         cocci_file.close()
+        # get version of spatch
+        if self.spatch_newer_than("1.0.0-rc6"):
+            cocci_op = "=~"
+        else:
+            cocci_op = "~="
+        # do substitution
         cocci_smpl = cocci_smpl_tmpl.substitute(type=self.type,
-            attribut=self.attribut)
+            attribute=self.attribute, cocci_regexp_equal=cocci_op)
         cocci_grep = cocci_smpl + CocciGrep.cocci_python
 
         tmp_cocci_file.write(cocci_grep)
